# lohrothek-api

The lohrothek-api project provides a django-based software for managing radio programs, series, and recordings. It implements a REST-API for implementing various frontends and automatically transcodes uploaded audio files in different formats for mobile clients.

---

# Setup for Production

We provide `deb` packages for Debian based operating systems. Please note, that this is currently a rather dirty package (only amd64, containing a virtualenv with some needed python3 dependencies). The project is hard under development, consider the packages as unstable but usually working. We recommend a setup using Debian Buster (or later), NGINX as webserver, uWSGI as appserver and PostgreSQL for the database.

Alternatives listet afterwards.

## Quick Setup | Using Debian Packages

First add the LOHRO Repository to your sources. Please notice that it is currently not signed.

```bash
echo "deb http://deb.lohro.de/ unstable main" > /etc/apt/sources.list.d/lohro.list
```

If you're using `buster` add the backports to get Django in the newer version 2. Add this line to your `/etc/apt/sources.list`...

```apt
deb http://deb.debian.org/debian buster-backports main
```

... then update the repos...

```bash
apt update
```

... and finally preinstall `python3-django` from the backports:

```bash
apt -t buster-backports install python3-django
```

Now install all packages. We added the default stuff in the suggestions, so just install:

```bash
apt install --install-suggests lohrothek-backend
```

The LOHROthek packages contain some default configurations. To setup the uWSGI we just enable the shipped config by linking...

```bash
cd /etc/uwsgi/apps-enabled/
ln -s ../apps-available/lohrothek.ini
```

Then we need nginx as webserver. The `lohrothek-backend`-package provides an example configuration

```bash
cp /usr/share/doc/lohrothek-backend/examples/nginx-example.conf /etc/nginx/sites-available/lohrothek.conf
```

Edit the new `/etc/nginx/sites-available/lohrothek.conf` set your hostname like

```nginx
server_name media.example.com;
```

The real magic happens in the also shipped and already placed `/etc/nginx/snippets/lohrothek.conf`. Please have a look inside to get some feelings about the nginx details.

Now enable the new nginx config by linking:

```sh
cd /etc/nginx/sites-enabled/
ln -s ../site-available/lohrothek.conf
```

Perhaps disable the default configuration linked in `sites-enabled` by deleting the link.

**We strongly recommend using SSL, e.g. by using [Let's Encrypt](https://letsencrypt.org/) with the `dehydrated`-package. Don't use a production system with http / port 80 / unencryped for security reasons** There are a lot of well working examples and tutorials in the web, we don't redocumentate it here.

Before we configure the database, we need to add the locales we want to use. Locales define some language-specific stuff like common sorting orders. First (if not already there) install and reconfigure the `locales`-package:

```bash
apt install locales
dpkg-reconfigure locales
```

In Germany enable the `de_DE.UTF-8` entry in the list and click ok. In Austria choose `de_AT.UTF-8`. In other countries choose your suitable favourite.

Now we setup the database. For PostgreSQL there is a user called `postgres` who is created by package-installation. This user has administrative permissions for the database configuration. You don't need a new database own superuser for the database, just `su` in him / her and open the PostgreSQL client.

```bash
su - postgres
psql
```

Add a new user and a new database by execute the following lines the opened `psql`-shell. For LC_COLLATE and LC_CTYPE use the locale you have choosen before, the following is for German installations. Also replace the password (!!!)

```sql
CREATE USER lohrothek with password 'YOURVERYSTRONGPASSWORDHERE';
CREATE DATABASE lohrothek WITH ENCODING 'UTF8' LC_COLLATE='de_DE.UTF8' LC_CTYPE='de_DE.UTF8' TEMPLATE=template0 OWNER lohrothek;
```

Close the PostgreSQL-shell and go further like before as `root` user.

Now we need to set some configuration settings for the LOHROthek backend. Copy the example to the expected place.

```sh
cp /etc/lohrothek/local_settings.py.dist /etc/lohrothek/local_settings.py
```

Then adapt the config to your needs.

-   Set a new random `SECRET_KEY`. Django needs it for some random- and crypto stuff. Generate an new key with `` pwgen -s -y 60 -N 1 -r \`\'\" ``. Keep this key absolulte secret!
-   Set `DEBUG = True` if you need some DEBUG information later in the browser while you setup the LOHROthek. But in production please keep it `False`.
-   Set your Hostname(s) with `ALLOWED_HOSTS=['media.example.com', 'audio.example.com']`. An empty list would allow all request regardless the domain, don't be lazy.
-   Set `HOST` (possibly "localhost") and `PASSWORD` (you defined before) for your PostgreSQL Database
-   Set `ADMIN` Mailcontact
-   Set `MEDIA_ROOT` to the path you want to store the mediafiles.

There are some further configuration-details:

-   ~~SSL~~ (coming soon)
-   ~~LDAP as user-authentication~~ (coming soon)
-   ~~Logging~~ (coming soon)
-   ~~FileStorage~~ (coming soon)
-   ~~Recording Service~~ (coming soon)

Now reload (or if possible: restart) all services we configured:

```bash
systemctl reload uwsgi.service postgresql.service nginx.service
```

We are almost there. The `lohrothek-backend` provides a small wrapper-script called `lohroctl` as helper for the django commands. First initialize the LOHROthek to create the database structure by:

```bash
lohroctl migrate
```

Then create the LOHROthek superuser

```bash
lohroctl createsuperuser
```

**Done! :)** Open the LOHROthek-backend in `lohrothek.example.com/manage/` and log in with out created administrator.

### Some Installation Debugging Goodies

Here are some notes who may help if the setup is not working properly.

First Try to restart all services and check, if some service fails with an error message.

```bash
systemctl reload uwsgi.service postgresql.service nginx.service
```

#### Logfiles

-   uWSGI in `/var/log/uwsgi/app/lohrothek.log`
-   nginx in `/var/log/nginx/...`
-   PostgreSQL in `/var/log/postgresql/...`
-   ~~LOHROthek in `/var/log/lohrothek/...`~~ (coming soon)

#### Database

Check if the login to your PostgreSQL is working by:

```bash
psql postgres://lohrothek@localhost:5432/lohrothek
```

Then list the existing databases in the `psql`-shell with `\l`, you should see the database 'lohrothek' with your prefered locales.

---

### Manual Deployment

If you prefer to deploy manually see the [deployment guide](doc/DEPLOYMENT.md).

---

### Alternative Production Setups

The recommended setup is not fixed and you can adapt it to your needs or existing enviroment. So try to migrate the nginx configuration to Apache2, or use a MariaDB instead of PostgreSQL as DBMS. If it works, we would be happy to get some additional documentation from you to publish them here :)

But in development we use the described setup and we will also just test our code on it.

---

# Development Setup

You’ll need the following packages to install the python requirements for this project.

```bash
# install required packages (Debian-based operating systems)
apt install libmagic1 python3-psycopg2 virtualenv nodejs npm libldap2-dev libsasl2-dev python3-dev build-essential
```

Then run:

```bash
virtualenv -p python3 --system-site-packages ~/.virtualenvs/lohrothek-api . ~/.virtualenvs/lohrothek-api/bin/activate
pip install -r requirements.txt
npm install && npm run build
cp doc/local_settings.py.development lohrothek/local_settings.py
python manage.py migrate
python manage.py runserver
```

### Semi-Optional Fixtures / Initial Data

There are some fixtures in the scripts folder for your convenience. Without them you would need to add some fields manually in the database. To delete the existing database and load the bootstrap the database with the fixtures run the following commands:

```bash
rm db.sqlite3
python manage.py migrate
python manage.py createsuperuser
python manage.py loaddata scripts/start_fixture.json
```

You can add your own fixtures with more interesting test data by running:

```bash
python manage.py dumpdata --indent 4 -e sessions -e auth -e admin -e contenttypes > scripts/your_fixture_name.json
```

## Testing your changes with Docker

The project comes with a [docker-compose](https://docs.docker.com/compose/) configuration. docker-compose allows you to run multiple different services like a database and a web-server in a controlled environment. We use it to build a package for Debian-based operating systems like we would on the continuous integration server and start a server with the lohrothek-api application.

You can start this process by running:

```bash
docker-compose build --build-arg django_secret=A_SECRET_OF_YOUR_LIKING && docker-compose up
```

**Note**: Be aware that because this process will build the deb package for our project the git working directory must be clean and all of your changes have to be committed (though not pushed).

Once the build is complete and the services have been started you can access the application by navigating your browser to `http://localhost:8086`. You can also access the database that is available with `psql postgresql://lohrothek:lohrothek@localhost:54320/lohrothek`. Files in `./media` relative to the project directory are available via the NGINX web-server and in case you want explore new Django settings you can do so by editing the `docker/backend/local_settings.py` configuration file.
