from program.models import Broadcast
from actstream.models import any_stream, Action
from django.shortcuts import render, get_object_or_404
from django.contrib.contenttypes.models import ContentType
from django_filters.views import FilterView
from activity.filters import BroadcastActivityFilter, ReportActivityFilter
from django.views.generic.base import View
from audiothek.models import RadioReport
from django.db.models import Q


def broadcast_activity(request, pk):
    b = get_object_or_404(Broadcast, pk=pk)
    res = any_stream(b)
    return render(request, 'activity/broadcast.html', {'actions': res, 'object': b})


# FilterView allows later filter for verbs, actors etc.
class BroadcastActivity(FilterView):
    filterset_class = BroadcastActivityFilter
    context_object_name = 'actions'
    paginate_by = 20
    template_name = 'activity/broadcasts.html'

    def get_queryset(self):

        ctype = ContentType.objects.get(app_label="program", model="broadcast")
        return Action.objects.filter(action_object_content_type_id=ctype)


class RadioReportActivity(View):

    def get(self, *args, **kwargs):
        b = get_object_or_404(RadioReport, pk=self.kwargs.get('pk'))
        res = any_stream(b)
        return render(self.request, 'activity/report.html', {'actions': res, 'object': b})


# FilterView allows later filter for verbs, actors etc.
class RadioReportListActivity(FilterView):
    filterset_class = ReportActivityFilter
    context_object_name = 'actions'
    paginate_by = 20
    template_name = 'activity/reports.html'

    def get_queryset(self):
        ctype = ContentType.objects.get(app_label="audiothek", model="radioreport")
        return Action.objects.filter(
            Q(target_content_type_id=ctype) | Q(action_object_content_type_id=ctype))
