from audiothek.models import RadioReport, RadioReportCategory
from django_filters import FilterSet
from django_filters.filters import CharFilter, DateFromToRangeFilter, \
    OrderingFilter, ChoiceFilter
from core.widgets import DateFromToRangeWidget, NameTaggitSelect2
from operator import and_
from functools import reduce
from django.db.models import Q
from django.db.models import Count
from audiothek.queryset import radioreport_is_public, radioreport_is_private
from core.fields import M2MTreeNodeSelectField
from django.http.request import QueryDict
import django_filters
from taggit.forms import TagField


YES_NO_CHOICES = (
    ('true', 'Ja'),
    ('false', 'Nein'),
)


class TagFilter(django_filters.CharFilter):
    field_class = TagField

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('lookup_expr', 'in')
        super().__init__(*args, **kwargs)


def icontains_stringtokenize(queryset, name, value):
    e = name + '__icontains'
    li = value.split()
    return queryset.filter(reduce(and_, [Q(**{e: q}) for q in li]))


class RadioReportFilter(FilterSet):

    def hasaudio_filter(queryset, name, value):

        if value == "true":
            return queryset.annotate(audiofilecount=Count('audiofiles')) \
                .filter(audiofilecount__gt=0)
        elif value == "false":
            return queryset.annotate(audiofilecount=Count('audiofiles')) \
                .filter(audiofilecount__exact=0)
        else:
            return queryset

    def ispublic_filter(queryset, name, value):

        if value == "true":
            return radioreport_is_public(queryset)

        elif value == "false":
            return radioreport_is_private(queryset)

        else:
            return queryset

    has_audio = ChoiceFilter(
        choices=YES_NO_CHOICES,
        method=hasaudio_filter,
        empty_label='egal',
        label='hat Audio')

    is_public = ChoiceFilter(
        choices=YES_NO_CHOICES,
        method=ispublic_filter,
        empty_label='egal',
        label='veröffentlicht')

    prod = DateFromToRangeFilter(
        'production_date',
        widget=DateFromToRangeWidget(),
        label='Produktionsdatum',
    )

    title = CharFilter(
        'title',
        label='Titel',
        method=icontains_stringtokenize
    )

    tags = TagFilter(label='Schlagwörter', field_name='tags__name', widget=NameTaggitSelect2(
                'tag-autocomplete', attrs={'data-theme': 'bootstrap4'}
            ), help_text="bei mehreren Schlagwörtern: ODER-Verknüpfung\
            (Beitrag muss eines der Schlagworte enthalten)")

    order = OrderingFilter(
        fields=(
            ('title', 'Titel'),
            ('production_date', 'Produktionsdatum'),
            ('creation_time', 'erstellt'),
            ('last_edit_time', 'zuletzt bearbeitet'),
        )
    )

    class Meta:
        model = RadioReport
        fields = {'title', 'production_date', 'categories', 'tags'}
        extra_fields = {'has_audio', 'is_public'}

    def __init__(self, data=None, *args, **kwargs):

        if data is None:
            data = QueryDict()

        if data.get('order') is None:
            data._mutable = True
            data.setdefault('order', '-Produktionsdatum')

        super(RadioReportFilter, self).__init__(data, *args, **kwargs)

        self.filters['categories'].field_class = M2MTreeNodeSelectField
        self.filters['categories'].queryset = RadioReportCategory.objects_report_count.all()
        self.filters['categories'].label = 'Themenfeld'
