import pytz
from lohrothek import settings
from datetime import datetime, timedelta
from django.db.models.expressions import Exists, OuterRef
from program.models import Broadcast


def radioreport_is_public(qs):

    now = pytz.timezone(settings.TIME_ZONE).localize(datetime.now())

    # subquery: broadcasts of report +/-7 days from now
    bc = Broadcast.objects.filter(items__report__pk=OuterRef('pk'))\
        .filter(start_time__gte=now-timedelta(7)).filter(start_time__lte=now+timedelta(7))

    # prefix query for all queries
    qs_all = qs.filter(publication_time__isnull=False).filter(publication_time__lte=now)

    # suffix query for persistent published radio reports
    qs_pers = qs_all.filter(publicationlimit__exact='')

    # suffix query for temporarily published radio reports
    qs_temp = qs_all.exclude(publicationlimit__exact='').annotate(b=Exists(bc)).filter(b=True)

    # returning OR query
    return qs_pers | qs_temp


def radioreport_is_private(qs):

    now = pytz.timezone(settings.TIME_ZONE).localize(datetime.now())

    # subquery: broadcasts of report +/-7 days from now
    bc = Broadcast.objects.filter(items__report__pk=OuterRef('pk'))\
        .filter(start_time__gte=now-timedelta(7)).filter(start_time__lte=now+timedelta(7))

    # reports set to private
    qs_priv = qs.filter(publication_time__isnull=True)

    # reports will be published in future
    qs_future = qs.filter(publication_time__isnull=False).filter(publication_time__gt=now)

    # reports with publication limit and no valid broadcast
    qs_temp = qs.filter(publication_time__isnull=False).filter(publication_time__lte=now)\
        .exclude(publicationlimit__exact='').annotate(b=Exists(bc)).filter(b=False)

    # returning OR query
    return qs_priv | qs_future | qs_temp
