from django.template.defaultfilters import slugify
from secrets import token_urlsafe
from lohrothek import settings

import os


def build_radioreport_changes(report, trackfields):
    changes = {}

    for trackfield in trackfields:

        if trackfield in report.tracker.changed().keys():

            changes[trackfield] = getattr(report, trackfield)

            if not changes[trackfield] and report.tracker.previous(trackfield) is None:
                del changes[trackfield]
                continue

            if trackfield in ['production_date']:
                changes[trackfield] = changes[trackfield].strftime("%Y-%m-%d")
    return changes


def build_audiofile_changes(audiofile, trackfields):
    changes = {}

    for trackfield in trackfields:

        if trackfield in audiofile.tracker.changed().keys():

            changes[trackfield] = getattr(audiofile, trackfield)

            if not changes[trackfield] and audiofile.tracker.previous(trackfield) is None:
                del changes[trackfield]
                continue

    return changes


def rename_audiofile_filename(instance):

    initial_path = instance.audiofile.path
    ext = instance.audiofile.path.split('.')[-1]
    new_filename = get_available_media_filename(settings.AUDIO_SUBFOLDER,
                                                build_audiofile_prefix(instance), ext)
    new_path = os.path.join(settings.MEDIA_ROOT, new_filename)

    os.rename(initial_path, new_path)

    # update audiofile with new filename
    instance.audiofile = new_filename
    instance.save()


def build_audiofile_prefix(instance):

    report_date = instance.report.production_date.strftime('%y%m%d')
    report_slug = slugify(instance.report.title)

    return '{}_{}-{}__{}'.format(report_date, instance.report.pk, instance.id, report_slug)


def get_available_media_filename(media_sub_path, file_title, file_extension):

    media_path = os.path.join(media_sub_path, '{}.{}'.format(file_title, file_extension))
    abs_path = os.path.join(settings.MEDIA_ROOT, media_path)

    # append digit to filename if file already exists
    while os.path.isfile(abs_path):
        media_path = os.path.join(media_sub_path,
                                  '{}__{}.{}'.format(file_title, token_urlsafe(4), file_extension))
        abs_path = os.path.join(settings.MEDIA_ROOT, media_path)
    return media_path


def move_previous_audiofile(instance, old_audiofile_path):

    if not instance.audiofile:
        return

    if os.path.isfile(os.path.join(settings.MEDIA_ROOT, old_audiofile_path)):
        ext = old_audiofile_path.split('.')[-1]
        file_name = get_available_media_filename(settings.AUDIO_OVERWRITTEN_SUBFOLDER,
                                                 build_audiofile_prefix(instance), ext)

        if not os.path.exists(
            os.path.join(settings.MEDIA_ROOT, settings.AUDIO_OVERWRITTEN_SUBFOLDER)
        ):
            os.mkdir(os.path.join(settings.MEDIA_ROOT, settings.AUDIO_OVERWRITTEN_SUBFOLDER))

        os.rename(os.path.join(settings.MEDIA_ROOT, old_audiofile_path),
                  os.path.join(settings.MEDIA_ROOT, file_name))
