from mptt.forms import TreeNodeMultipleChoiceField
from core.widgets import M2MSelect
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.utils.encoding import smart_str

"""
A field to allow only single select on many to many relations between a model class
relating to a mptt  https://github.com/django-mptt

assuming count of related model instance to display is annotated by model manager
to attribute 'related_count'
"""


class M2MTreeNodeSelectField(TreeNodeMultipleChoiceField):
    widget = M2MSelect

    # add empty label
    def __init__(self, queryset, *args, **kwargs):
        super().__init__(queryset, *args, **kwargs)
        self.empty_label = '---'

    # allow empty value on query set
    def clean(self, value):
        value = self.prepare_value(value)
        if value:
            if (len(value) == 1) & (value[0] == ''):
                return self.queryset.none()
        return super().clean(value)

    def label_from_instance(self, obj):

        level_indicator = self._get_level_indicator(obj)

        try:
            return mark_safe(level_indicator + ' ' + conditional_escape(smart_str(obj))
                             + ' (%s)' % obj.related_count)
        except AttributeError:
            return mark_safe(level_indicator + ' ' + conditional_escape(smart_str(obj)))
