from django_filters.filterset import FilterSet
from _functools import reduce
from _operator import and_
from django.db.models import Q
from core.models import Department, DepartmentRole
from django_filters.filters import CharFilter, ModelChoiceFilter, ChoiceFilter
from program.models import RadioProgram
from django.contrib.auth.models import User


MEMBER_STATUS_CHOICES = (
    ('active', 'aktiv'),
    ('former', 'ehemalig'),
)


def icontains_stringtokenize(queryset, name, value):
    e = name + '__icontains'
    li = value.split()
    return queryset.filter(reduce(and_, [Q(**{e: q}) for q in li]))


class DepartmentFilter(FilterSet):

    programs = ModelChoiceFilter(queryset=RadioProgram.objects.all(), label='Sendereihe')

    name = CharFilter(
        'name',
        label='Name',
        method=icontains_stringtokenize
    )

    class Meta:
        model = Department
        fields = {'name', 'programs'}


class DepartmentRoleFilter(FilterSet):

    def memberstatus_filter(queryset, name, value):
        if value == 'active':
            return queryset.filter(end_date__isnull=True)
        elif value == 'former':
            return queryset.filter(end_date__isnull=False)
        return queryset

    status = ChoiceFilter(
        choices=MEMBER_STATUS_CHOICES,
        method=memberstatus_filter,
        empty_label='egal',
        label='Status')

    class Meta:
        model = DepartmentRole
        fields = {'name', 'user'}
        extra_fields = {'status'}

    def __init__(self, data=None, *args, **kwargs):
        d = kwargs.pop('department')
        super(DepartmentRoleFilter, self).__init__(data, *args, **kwargs)
        self.queryset = self.queryset.filter(department=d)
        self.filters['name'].label = "Rolle"
        self.filters['user'].label = "Benutzername"
        self.filters['user'].queryset = User.objects.filter(departmentroles__department=d)\
            .order_by('first_name')


class UserFilter(FilterSet):

    def username_filter(queryset, name, value):
        return queryset.filter(Q(username__icontains=value) |
                               Q(first_name__icontains=value) |
                               Q(last_name__icontains=value) |
                               Q(profile__nickname__icontains=value))

    name = CharFilter(
        'name',
        method=username_filter,
        label='Name'
    )

    class Meta:
        model = User
        fields = {'username'}
        extra_fields = {'name'}

    def __init__(self, data=None, *args, **kwargs):
        super(UserFilter, self).__init__(data, *args, **kwargs)
