import os

from django.db.models.fields.files import FileField
from django.http import HttpResponse
from django.utils.text import slugify


class HttpResponseSendfile(HttpResponse):
    def __init__(self, file_field: FileField, *args, filename=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_field = file_field
        url = file_field.url.encode('utf-8')
        self['X-Accel-Redirect'] = url
        self['X-Sendfile'] = url
        if filename:
            _, extension = os.path.splitext(file_field.name)
            # we don’t want unicode in the attachment filename as
            # support for them is very weird and (see RFC 5987)
            # and may not even work in every client
            attachment_name = slugify(filename, False) + extension
            self['Content-Disposition'] = 'attachment; filename="{}"'.format(attachment_name)
        # Web servers (in our case NGINX) will pass the Content-Type header provided
        # by our application back to the client. But its the web server that should
        # figure out the proper Content-Type via its static file handler.
        del self['Content-Type']
