import re
from django.utils.formats import get_format
from datetime import datetime
from urllib.parse import urlparse
from django.forms.models import modelform_factory


def get_referer_view(request, default=None, removeGet=False):
    '''
    Return the referer view of the current request

    Example:

        def some_view(request):
            ...
            referer_view = get_referer_view(request)
            return HttpResponseRedirect(referer_view, '/accounts/login/')
    '''

    # if the user typed the url directly in the browser's address bar
    referer = request.META.get('HTTP_REFERER')

    if not referer:
        return default

    if request.META.get('SERVER_NAME') not in ['localhost', '127.0.0.1']:
        if referer[0] != request.META.get('SERVER_NAME'):
            return default

    if removeGet:
        referer = urlparse(referer).path

    # remove the protocol and split the url at the slashes
    referer = re.sub(r'^https?:\/\/', '', referer).split('/')

    # add the slash at the relative path's view and finished
    referer = u'/' + u'/'.join(referer[1:])

    return referer


def parse_date(date_str):
    """Parse date from string by DATE_INPUT_FORMATS of current language"""
    for item in get_format('DATE_INPUT_FORMATS'):
        try:
            return datetime.strptime(date_str, item).date()
        except (ValueError, TypeError):
            continue

    return None


class ModelFormWidgetMixin(object):
    def get_form_class(self):
        return modelform_factory(self.model, fields=self.fields, widgets=self.widgets)
