from django_filters.widgets import RangeWidget
from django.forms import DateInput
from django.utils.html import mark_safe
from django.forms import Select
from django.utils.datastructures import MultiValueDict
from dal_select2_taggit.widgets import TaggitSelect2


"""
Modified Taggit Select widget for django-filter to lookup for name
and display name only, otherwise recursive list building
"""


class NameTaggitSelect2(TaggitSelect2):

    def option_value(self, value):
        res = super().option_value(value)
        if type(res) is list:
            return res[0]
        return res


"""
A widget to allow only single select on many to many relations
"""


class M2MSelect(Select):
    def value_from_datadict(self, data, files, name):
        if isinstance(data, MultiValueDict):
            return data.getlist(name)
        return data.get(name, None)


class DateFromToRangeWidget(RangeWidget):

    def __init__(self, attrs=None):
        widgets = (DateInput, DateInput)
        super(RangeWidget, self).__init__(widgets, attrs)

    def render(self, name, value, attrs=None, renderer=None):
        """Copy and past from original render method"""
        if self.is_localized:
            for widget in self.widgets:
                widget.is_localized = self.is_localized

        if not isinstance(value, list):
            value = self.decompress(value)
        output = []
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        for i, widget in enumerate(self.widgets):
            try:
                widget_value = value[i]
            except IndexError:
                widget_value = None
            if id_:
                final_attrs = dict(final_attrs, id='%s_%s' % (id_, i))
            output.append(widget.render(name + '_%s' % self.suffixes[i],
                                        widget_value, final_attrs))
        return [mark_safe(self.format_value(x)) for x in output]
