# Erstmaliges Deployment (Debian und Derivate)

## Paketquellen für Node.js hinzufügen

In Debian Stretch muss das [`stretch-backports`](https://backports.debian.org/Instructions/) Repository hinzugefügt werden. In Debian Buster reichen die offiziellen Paketquellen aus. Anschließend müssen die Pakete `nodejs` und `npm` installiert werden.

Auf anderen Systemen lassen sich ggf. auch die offiziellen Quellen von NodeJS hinzufügen. Nähere Informationen dazu finden sich [auf der entsprechenden Info-Seite](https://nodejs.org/en/download/package-manager/).

## Pakete installieren

    apt-get install libmagic1 nginx-light python3-psycopg2 uwsgi uwsgi-plugin-python3 virtualenv
    apt-get install nodejs  # für Node.js (npm)
    apt-get install libldap2-dev libsasl2-dev python3-dev gcc  # für django-auth-ldap

## Repository auschecken

    cd /usr/local/share
    git clone git@git.hack-hro.de:lohro/lohrothek/lohrothek-api.git

## Virtualenv erstellen

    virtualenv --python=/usr/bin/python3 --system-site-packages /usr/local/share/lohrothek-api/virtualenv
    source /usr/local/share/lohrothek-api/virtualenv/bin/activate
    pip install -r /usr/local/share/lohrothek-api/requirements.txt

## Kontrollskript und Konfigurationsdatei installieren

    useradd lohrothek
    ln -s /usr/local/share/lohrothek-api/scripts/lohroctl /usr/local/bin
    mkdir /etc/lohrothek
    cp /usr/local/share/lohrothek-api/doc/local_settings.py.production /etc/lohrothek/local_settings.py
    ln -s /etc/lohrothek/local_settings.py /usr/local/share/lohrothek-api/lohrothek/local_settings.py

## Datenbank einrichten

Datenbankanbindung in `/etc/lohrothek/local_settings.py` konfigurieren. Die Datenbank sollte das Locale `de_DE.UTF-8` verwenden.

    lohroctl migrate
    
## Datenbank Fixutres anwenden

Programmmodul: Sendungsreihen, -Webseitenrreferenzen, Sendungstemplates

    lohroctl loaddata scripts/program_fixture.json


## uWSGI einrichten

    cp /usr/local/share/lohrothek-api/doc/uwsgi.ini /etc/uwsgi/apps-available/lohrothek.ini
    ln -s ../apps-available/lohrothek.ini /etc/uwsgi/apps-enabled
    service uwsgi restart

## Statische Dateien einsammeln

    mkdir /usr/local/share/lohrothek-api/static
    chown lohrothek: /usr/local/share/lohrothek-api/static
    cd /usr/local/share/lohrothek-api/static && npm install && npm run build
    lohroctl collectstatic

## nginx einrichten

    mkdir -p /var/local/lib/lohrothek/media
    chown www-data:lohrothek /var/local/lib/lohrothek/media
    chmod g+w /var/local/lib/lohrothek/media
    ln -s /var/local/lib/lohrothek/media /usr/local/share/lohrothek-api

    cp /usr/local/share/lohrothek-api/doc/nginx-example.conf /etc/nginx/sites-available/lohrothek
    (cd /etc/nginx/sites-enabled && ln -s ../sites-available/lohrothek)

Konfiguration in `/etc/nginx/sites-available/lohrothek` anpassen und anschließend nginx neustarten (`service nginx restart`).


# Update

## Repository aktualisieren

    cd /usr/local/share/lohrothek-api
    git pull

## Virtualenv aktualisieren

    source /usr/local/share/lohrothek-api/virtualenv/bin/activate
    pip install -U -r /usr/local/share/lohrothek-api/requirements.txt

## Datenbank aktualisieren

    lohroctl migrate

## Statische Dateien aktualisieren

    npm install && npm run build
    lohroctl collectstatic

## Änderungen aktivieren

    touch /etc/lohrothek/local_settings.py
    service nginx reload

Alternativ `service uwsgi reload` ausführen.
