"""
Django settings for lohrothek project.

Generated by 'django-admin startproject' using Django 2.1.

For more information on this file, see
https://docs.djangoproject.com/en/2.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.1/ref/settings/
"""

import os
from django.contrib.messages import constants as messages

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Application definition

INSTALLED_APPS = [
    'activity',
    'audiothek',
    'core',
    'corsheaders',
    'program',
    'moderation',
    'rest_framework',
    'django_filters',
    'taggit',
    'dal',
    'dal_select2',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    'django.forms',
    'actstream',
    'flatpickr',
    'huey.contrib.djhuey',
    'martor',
    'mutagen',
    'mptt',
    'imagekit'
]


# only add django_extensions if pydotplus is installed to minimize dependencies
# this is only needed to autogenerate UML from Django Modells
# with python manage.py graph_models -a -o lohrothek_modell.png
try:
    import pydotplus  # noqa: F401
    INSTALLED_APPS.append('django_extensions')
except ImportError:
    pass


MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'core.middleware.LoginRequiredMiddleware',
    'core.middleware.UserBasedExceptionMiddleware',
]

ROOT_URLCONF = 'lohrothek.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'builtins': ['core.templatetags.core'],
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'lohrothek.context_processors.public_settings_vars',
            ],
        },
    },
]

FORM_RENDERER = 'django.forms.renderers.TemplatesSetting'

WSGI_APPLICATION = 'lohrothek.wsgi.application'

MESSAGE_TAGS = {
    messages.DEBUG: 'alert-info',
    messages.INFO: 'alert-info',
    messages.SUCCESS: 'alert-success',
    messages.WARNING: 'alert-warning',
    messages.ERROR: 'alert-danger',
}

# Password validation
# https://docs.djangoproject.com/en/2.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.1/topics/i18n/

LANGUAGE_CODE = 'de-de'

TIME_ZONE = 'Europe/Berlin'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Media files (uploaded files)
#

AUDIO_SUBFOLDER = 'audio'
AUDIO_OVERWRITTEN_SUBFOLDER = 'audio_overwritten'

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

MEDIA_URL = '/manage/media/'


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.1/howto/static-files/

STATIC_URL = '/manage/static/'

STATIC_ROOT = os.path.join(BASE_DIR, 'static')


# Registration

LOGIN_URL = '/manage/accounts/login/'

LOGIN_REDIRECT_URL = LOGOUT_REDIRECT_URL = 'home'


# Django REST framework
# https://www.django-rest-framework.org/

REST_FRAMEWORK = {
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),
    'DEFAULT_PAGINATION_CLASS': 'rest_api.frontend.util.CustomPageNumberPagination',
}


# Flatpickr settings
# https://bootstrap-datepicker.readthedocs.io/

FLATPICKR_SETTINGS = {
    'options': {
        'locale': 'de',
        'dateFormat': 'd.m.Y H:i',
        'time_24hr': True,
    }
}


# Actstream settings
# https://django-activity-stream.readthedocs.io/

SITE_ID = 1
ACTSTREAM_SETTINGS = {
    'FETCH_RELATIONS': True,
    'USE_JSONFIELD': True,
}

# Martor settings
# https://github.com/agusmakmun/django-markdown-editor

MARTOR_ENABLE_CONFIGS = {
    'imgur': 'false',     # to enable/disable imgur/custom uploader.
    'mention': 'false',  # to enable/disable mention
    'jquery': 'true',    # to include/revoke jquery (require for admin default django)
    'living': 'false',   # to enable/disable live updates in preview
    'spellcheck': 'true',
}


# LOHROthek settings
#

LOHROTHEK_SUPPORT_EMAIL = 'lohrothek@lohro.de'

LOHROTHEK_RECORDING_API_URL = 'http://localhost:8001/snippets/'

LOHROTHEK_FFMPEG = '/usr/bin/ffmpeg'
# see audiothek.processing.EncodingFormats for possible options
LOHROTHEK_TRANSCODE_FORMATS = ['MP4_AAC', 'OGG_OPUS', 'MP3']

APPEND_SLASH = False

CORS_ORIGIN_ALLOW_ALL = True
# Add CORS headers for all routes starting with either /api/ or /media/.
# Note that the /media/ path is only handled in dev-mode as static resources are delivered
# by a proper web server in production and must be configured separately.
CORS_URLS_REGEX = r'^/(api|media)/.*$'
CORS_ALLOW_METHODS = ('GET', 'HEAD', 'OPTIONS')


try:
    from lohrothek.local_settings import *  # noqa: 401
except ImportError:
    pass
