from operator import and_
from functools import reduce
from django.db.models import Q
from django_filters.filterset import FilterSet
from django_filters.filters import CharFilter, OrderingFilter,\
    DateFromToRangeFilter
from moderation.models import Moderation
from core.widgets import DateFromToRangeWidget


def mod_icontains_stringtokenize(queryset, name, value):

    li = value.split()

    modname = name + '__icontains'
    qs_modname = queryset.filter(reduce(and_, [Q(**{modname: q}) for q in li]))

    reporttitle = 'report__title__icontains'
    qs_reporttitle = queryset.filter(reduce(and_, [Q(**{reporttitle: q}) for q in li]))

    return qs_modname | qs_reporttitle


class ModerationFilter(FilterSet):

    title = CharFilter(
        'name',
        label='Titel',
        method=mod_icontains_stringtokenize
    )

    created = DateFromToRangeFilter(
        'creation_time',
        widget=DateFromToRangeWidget(),
        label='Erstelldatum',
    )

    order = OrderingFilter(
        fields=(
            ('name', 'Titel'),
            ('creation_time', 'Erstellungsdatum'),
        )
    )

    class Meta:
        model = Moderation
        fields = {'name', 'creation_time'}

    def __init__(self, data=None, *args, **kwargs):

        if data is None:
            data = {'order': '-Erstellungsdatum'}
        super(ModerationFilter, self).__init__(data, *args, **kwargs)
