from django.urls.conf import path
from moderation.views import ModerationList, ModerationCreate, ModerationDetail,\
    ModerationUpdate, ModerationPrintDetail, ModerationDelete,\
    ModerationItemSpeakCreate, ModerationItemSpeakUpdate,\
    ModerationItemSystemCreate, ModerationItemSystemUpdate, ModerationItemDelete,\
    move_moderation_item, moderation_remove_report, ModerationSelectReportList,\
    moderation_add_report, ModerationSelectBroadcastList,\
    quick_add_broadcast_to_moderation

urlpatterns = [

    path('moderations/', ModerationList.as_view(), name='list-moderations'),
    path('moderations/create/', ModerationCreate.as_view(), name='create-moderation'),
    path('moderations/create/report/<int:report>/',
         ModerationCreate.as_view(), name='create-moderation-for-report'),
    path('moderations/create/broadcast/<int:broadcast>/',
         ModerationCreate.as_view(), name='create-moderation-for-broadcast'),

    path('moderations/<int:pk>/', ModerationDetail.as_view(), name='show-moderation'),
    path('moderations/<int:pk>/edit/', ModerationUpdate.as_view(), name='update-moderation'),
    path('moderations/<int:pk>/print/', ModerationPrintDetail.as_view(), name='print-moderation'),
    path('moderations/<int:pk>/delete/', ModerationDelete.as_view(), name='delete-moderation'),

    path('moderations/<int:moderation_pk>/remove-report/',
         moderation_remove_report, name='remove-report-from-moderation'),
    path('moderations/<int:moderation_pk>/select-report/',
         ModerationSelectReportList.as_view(), name='select-report-for-moderation'),
    path('moderations/<int:moderation_pk>/add-report/<int:report_pk>',
         moderation_add_report, name='add-report-to-moderation'),

    path(
        'moderations/<int:moderation_pk>/broadcast/<int:broadcast_pk>/quickadd/',
        quick_add_broadcast_to_moderation,
        name='quick-add-broadcast-to-moderation'),
    path('moderations/<int:moderation_pk>/select-broadcast/',
         ModerationSelectBroadcastList.as_view(), name='select-broadcast-for-moderation'),

    path('moderations/<int:pk>/add-speak/',
         ModerationItemSpeakCreate.as_view(), name='create-moderation-item-speak'),
    path('moderation-item/speak/<int:pk>/edit/',
         ModerationItemSpeakUpdate.as_view(), name='edit-moderation-item-speak'),

    path('moderations/<int:pk>/add-system/',
         ModerationItemSystemCreate.as_view(), name='create-moderation-item-system'),
    path('moderation-item/system/<int:pk>/edit/',
         ModerationItemSystemUpdate.as_view(), name='edit-moderation-item-system'),

    path('moderation-item/<int:pk>/delete/',
         ModerationItemDelete.as_view(), name='delete-moderation-item'),

    path('moderation-item/<int:pk>/move/',
         move_moderation_item, name='move-moderation-item'),
]
