from enum import Enum
from core.models import django_enum
from django.utils.timezone import localtime
from operator import attrgetter
from datetime import time


@django_enum
class BroadcastIrregularity(Enum):
        overlap = "Überlappung"
        empty_between = "Leerlauf zwischen"
        empty_before = "Leerlauf vor"
        empty_after = "Leerlauf nach"

        def __repr__(self):
            return self.name

        def __str__(self):
            return self.value


def check_program_irregularity_day(broadcasts, warnings=True):

    if not broadcasts:
        return []

    out = []

    if warnings:
        # only one programm@day: check if it start at midnight
        if len(broadcasts) == 1 and localtime(broadcasts[0].start_time).time() > time(0, 0):
                out.append((BroadcastIrregularity.empty_before, broadcasts[0]))

        # only one programm@day: check if it start at midnight and does not span over days
        elif len(broadcasts) > 1 and \
            localtime(broadcasts[0].start_time).date() == \
            localtime(broadcasts[0].end_time).date() and \
            localtime(broadcasts[0].start_time).time() > time(0, 0): # noqa
                out.append((BroadcastIrregularity.empty_before, broadcasts[0]))

    for i, bc in enumerate(broadcasts, start=0):

        # check time overlapping with broadcasts after current one
        overlaps = filter(lambda x:
                          localtime(x.start_time) < localtime(bc.end_time) and
                          localtime(x.end_time) > localtime(bc.start_time),
                          list(y for j, y in enumerate(broadcasts) if j < i)
                          )
        for ol in overlaps:
            out.append((BroadcastIrregularity.overlap, ol, bc))

        # check for empty time period with next broadcast (min start time)
        if warnings:
            bc_next = min(list(y for j, y in enumerate(broadcasts) if j > i) or [bc],
                          key=attrgetter('start_time'))
            if bc != bc_next and (bc_next.start_time - bc.end_time).total_seconds() > 60:
                out.append((BroadcastIrregularity.empty_between, bc, bc_next))

    if warnings:
        # check last element if it ends before midnight does not span over days
        if localtime(broadcasts[len(broadcasts)-1].start_time).date() == \
           localtime(broadcasts[len(broadcasts)-1].end_time).date() and \
           localtime(broadcasts[len(broadcasts)-1].end_time).time() < time(23, 59):
            out.append((BroadcastIrregularity.empty_after, broadcasts[len(broadcasts)-1]))

    return out
