from program.models import Broadcast, BroadcastTemplate, WEEKDAYS_BITCHOICE
from django_filters import FilterSet
from django_filters.filters import DateFilter, ChoiceFilter
from datetime import date
from program.querysets import query_broadcasts_of_day
from program.broadcast_transform import adjust_gapfill_broadcasts


class BroadcastFilter(FilterSet):

    def at_day(queryset, name, value):

        return query_broadcasts_of_day(queryset, value)

    date = DateFilter(
        'start_time',
        label='Datum',
        method=at_day,
    )

    class Meta:
        model = Broadcast
        fields = {}
        extra_fields = {'date'}

    def __init__(self, data=None, *args, **kwargs):

        if data is None:
            data = {'date': date.today()}
        super(BroadcastFilter, self).__init__(data, *args, **kwargs)

    def filter_queryset(self, queryset):
        return adjust_gapfill_broadcasts(super(BroadcastFilter, self).filter_queryset(queryset))


class BroadcastTemplateFilter(FilterSet):

    def has_weekday(queryset, name, value):

        if value != "":
            return queryset.extra(where=['weekdays & {} > 0'.format(value)])
        return queryset

    hasweekday = ChoiceFilter(
        choices=WEEKDAYS_BITCHOICE._choices,
        method=has_weekday,
        empty_label='- alle -',
        label='Wochentag')

    class Meta:
        model = BroadcastTemplate
        fields = {'program', 'weekrepeat'}
        extra_fields = {'hasweekday'}

        def __init__(self, data=None, *args, **kwargs):
            super(BroadcastTemplateFilter, self).__init__(data, *args, **kwargs)
            if self.data == {}:
                self.queryset = self.queryset.none()
