from datetime import date, timedelta, datetime
from math import ceil
from program.models import Broadcast, BroadcastTemplate, BroadcastRepeatTemplate,\
    BroadcastItem
from django.db.models import Q
from functools import reduce


def last_day_of_month(d):

    if not isinstance(d, date):
        raise TypeError("No valid day data type provided")

    if d.month == 12:
        lastday = 31
    else:
        lastday = (d.replace(month=d.month+1, day=1) - timedelta(days=1)).day

    return lastday


def repeat_broadcast_from_template(repeattmpl: BroadcastRepeatTemplate, firstbc: Broadcast):

    daystoadd = repeattmpl.plusweek * 7 + repeattmpl.weekday - firstbc.start_time.weekday()
    if repeattmpl.weekday - firstbc.start_time.weekday() < 0:
        daystoadd += 7

    repeat_date = firstbc.start_time.date() + timedelta(days=daystoadd)
    duration_time = (firstbc.end_time - firstbc.start_time)

    bc = Broadcast()
    bc.firstrun = firstbc
    bc.program = repeattmpl.origin.program
    bc.start_time = datetime.combine(repeat_date, repeattmpl.start_time)
    bc.end_time = bc.start_time + duration_time
    bc.gapfill = firstbc.gapfill
    return bc


def broadcast_from_template(tmpl: BroadcastTemplate, d: date):

    bc = Broadcast()
    bc.program = tmpl.program
    bc.start_time = datetime.combine(d, tmpl.start_time)
    bc.end_time = datetime.combine(d, tmpl.end_time)
    bc.gapfill = tmpl.gapfill
    return bc


def create_broadcast_regularitems_from_template(t: BroadcastTemplate, b: Broadcast):

    # bctmplri = broadcast template regular item
    for bctmplri in t.broadcasttemplateregularitems.all():
        bcitem = BroadcastItem()
        bcitem.broadcast = b
        bcitem.time = bctmplri.time
        bcitem.regular = bctmplri.regularitem
        bcitem.position = bctmplri.position
        bcitem.public = bctmplri.public
        bcitem.save()


def query_broadcast_templates_of_day(querydate):

    if not isinstance(querydate, date):
        raise TypeError("No valid date data type provided")

    monthweek = ceil(querydate.day / 7)
    weekday = querydate.weekday()
    is_odd_week = bool(querydate.isocalendar()[1] % 2)
    is_last_weekday_of_month = bool(last_day_of_month(querydate) - querydate.day < 7)

    q = BroadcastTemplate.objects.extra(where=['weekdays & %s > 0'], params=[2**weekday])

    wrq = []  # week repeat queries
    wrq.append(BroadcastTemplate.WEEKLY)

    if(monthweek == 1):
        wrq.append(BroadcastTemplate.MONTH_WEEK_1)
    elif(monthweek == 2):
        wrq.append(BroadcastTemplate.MONTH_WEEK_2)
    elif(monthweek == 3):
        wrq.append(BroadcastTemplate.MONTH_WEEK_3)
    elif(monthweek == 4):
        wrq.append(BroadcastTemplate.MONTH_WEEK_4)

    if(is_last_weekday_of_month):
        wrq.append(BroadcastTemplate.MONTH_WEEK_LAST)

    if(is_odd_week):
        wrq.append(BroadcastTemplate.BIWEEKLY_UNEVEN)
    else:
        wrq.append(BroadcastTemplate.BIWEEKLY_EVEN)

    return q.filter(reduce(lambda q, v: q | Q(weekrepeat=v), wrq, Q()))
