# inspired from
# https://stackoverflow.com/questions/3663898/representing-a-multi-select-field-for-weekdays-in-a-django-model/8152717#8152717


class BitChoices(object):

    def __init__(self, choices):
        self._choices = []
        self._lookup = {}
        for index, (key, val) in enumerate(choices):
            index = 2**index
            self._choices.append((index, val))
            self._lookup[key] = index

    def __iter__(self):
        return iter(self._choices)

    def __len__(self):
        return len(self._choices)

    def __getattr__(self, attr):
        try:
            return self._lookup[attr]
        except KeyError:
            raise AttributeError(attr)

    def get_selected_keys(self, selection):
        """ Return a list of keys for the given selection """

        if not selection:
            return

        c = []
        for k, b in self._lookup.items():
            if b & int(selection):
                c.append(b)
        return c

    def get_selected_values(self, selection):
        """ Return a list of values for the given selection """

        if not selection:
            return 0

        c = []
        for b, v in self._choices:
            if isinstance(selection, int):
                if b & int(selection):
                    c.append(v)
            else:
                for s in selection:
                    if b & int(s):
                        c.append(v)
        return c
