from django.urls import reverse

from core.tests import TestCase
from program.models import Broadcast, RadioProgram
from django.contrib.auth.models import User


class ProgramMixin:
    def setUp(self):
        super().setUp()
        self.program = RadioProgram.objects.create(name='Test Program')


class BroadcastTests(ProgramMixin, TestCase):
    def test_broadcast_creation_and_update(self):
        create_url = reverse('create-broadcast')
        list_url = reverse('list-broadcasts')
        test_description = 'Test Description'
        updated_test_description = 'Updated Test Description'

        username = 'John Doe'
        password = 'secret'
        User.objects.create_user(username=username, password=password, is_staff=True)
        self.client.login(username=username, password=password)

        # test broadcast creation
        r = self.client.get(list_url)
        if r.status_code == 302:
            r = self.client.get(r.url)

        self.assertNotContains(r, test_description)
        self.assertContains(r, 'href="{}"'.format(create_url))

        r = self.client.get(create_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(create_url, {
            'program': self.program.pk, 'start_time': '2000-01-01 00:00',
            'end_time': '2000-01-01 23:59', 'description': test_description})
        q = Broadcast.objects.filter(description=test_description)
        self.assertTrue(q.exists())
        self.assertRedirects(r, q.get().get_absolute_url())

        # FIXME: apply date filter to result list
        # r = self.client.get(list_url)
        # self.assertContains(r, test_description)

        # test broadcast update
        self.broadcast = Broadcast.objects.get(description=test_description)
        detail_url = self.broadcast.get_absolute_url()
        update_url = reverse('update-broadcast', args={self.broadcast.pk})

        r = self.client.get(detail_url)
        self.assertContains(r, update_url)

        r = self.client.get(update_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(update_url, {
            'program': self.program.pk, 'start_time': '2000-01-01 00:00',
            'end_time': '2000-01-01 23:59', 'description': updated_test_description})
        self.assertRedirects(r, detail_url)

        # FIXME: apply date filter to result list
        # r = self.client.get(list_url)
        # self.assertContains(r, updated_test_description)
