from django.urls import path

from program.views import BroadcastCreate, BroadcastDetail, \
    BroadcastList, BroadcastUpdate, BroadcastRepeatTemplateUpdate,\
    createBroadcastsFromTemplate, BroadcastDelete, BroadcastRepeatCreate,\
    BroadcastRepeatUpdate, BroadcastRepeatDelete, deleteBroadcastsByDates,\
    RadioProgramList, RadioProgramDetail, BroadcastRadioReportSelect, BroadcastItemReportCreate,\
    quick_add_report_to_broadcast, move_broadcast_item, BroadcastItemDelete,\
    BroadcastItemUpdate, BroadcastModerationSelect,\
    quick_add_moderation_to_broadcast, BroadcastItemModerationCreate,\
    BroadcastTemplateRegularItemCreate, BroadcastTemplateRegularItemUpdate,\
    BroadcastTemplateRegularItemDelete, move_regular_item_of_broadcast_template,\
    BroadcastItemRegularUpdate, BroadcastItemRegularCreate,\
    BroadcastTemplateCreate, BroadcastTemplateList, \
    BroadcastTemplateDetail, BroadcastTemplateUpdate, BroadcastTemplateDelete,\
    BroadcastRepeatTemplateCreate, BroadcastRepeatTemplateDelete
from program.views.broadcastitems import order_items_of_broadcast_by_time
from program.views.broadcasts import BroadcastRoleDelete

urlpatterns = [
    path('broadcasts/',
         BroadcastList.as_view(),
         name='list-broadcasts'),
    path('broadcasts/add/',
         BroadcastCreate.as_view(),
         name='create-broadcast'),
    path('broadcasts/<int:pk>/',
         BroadcastDetail.as_view(),
         name='show-broadcast'),
    path('broadcasts/<int:pk>/edit/',
         BroadcastUpdate.as_view(),
         name='update-broadcast'),
    path('broadcasts/<int:pk>/delete/',
         BroadcastDelete.as_view(),
         name='delete-broadcast'),
    path('broadcasts/bulk-delete/',
         deleteBroadcastsByDates,
         name='delete-bulk-broadcasts'),

    path('broadcasts/<int:pk>/delete-role/',
         BroadcastRoleDelete.as_view(),
         name='delete-broadcast-role'),

    path('broadcasts/<int:pk>/add-repeat/',
         BroadcastRepeatCreate.as_view(),
         name='create-broadcast-repeat'),
    path('broadcast-repeat/<int:pk>/edit',
         BroadcastRepeatUpdate.as_view(),
         name='update-broadcast-repeat'),
    path('broadcast-repeat/<int:pk>/delete',
         BroadcastRepeatDelete.as_view(),
         name='delete-broadcast-repeat'),

    path(
        'broadcasts/<int:broadcast_pk>/report/<int:report_pk>/quickadd/',
        quick_add_report_to_broadcast,
        name='quick-add-report-to-broadcast'),
    path(
        'broadcasts/<int:broadcast_pk>/add-report/',
        BroadcastRadioReportSelect.as_view(),
        name='select-report-for-broadcast'),
    path(
        'broadcasts/<int:broadcast_pk>/report/<int:report_pk>/select/',
        BroadcastItemReportCreate.as_view(),
        name='add-report-to-broadcast'),

    path(
        'broadcasts/<int:broadcast_pk>/moderation/<int:moderation_pk>/quickadd/',
        quick_add_moderation_to_broadcast,
        name='quick-add-moderation-to-broadcast'),
    path(
        'broadcasts/<int:broadcast_pk>/add-moderation/',
        BroadcastModerationSelect.as_view(),
        name='select-moderation-for-broadcast'),
    path(
        'broadcasts/<int:broadcast_pk>/moderation/<int:moderation_pk>/select/',
        BroadcastItemModerationCreate.as_view(),
        name='add-moderation-to-broadcast'),

    path('broadcasts/<int:broadcast_pk>/add-regular-item/',
         BroadcastItemRegularCreate.as_view(), name='add-regular-item-to-broadcast'),
    path('broadcasts/<int:pk>/order-items-by-time',
         order_items_of_broadcast_by_time, name='order-broadcast-items-by-time'),

    path('broadcast-item/<int:bcitem_pk>/move/<action>',
         move_broadcast_item, name='move-broadcast-item'),
    path('broadcast-item/<int:pk>/edit/',
         BroadcastItemUpdate.as_view(), name='edit-broadcast-item'),
    path('broadcast-item/<int:pk>/delete/',
         BroadcastItemDelete.as_view(), name='delete-broadcast-item'),

    path('broadcast-regular-item/<int:pk>/edit/',
         BroadcastItemRegularUpdate.as_view(), name='edit-regular-broadcast-item'),

    path('broadcast-templates/',
         BroadcastTemplateList.as_view(),
         name='list-broadcast-templates'),
    path('broadcast-templates/add/',
         BroadcastTemplateCreate.as_view(),
         name='create-broadcast-template'),
    path('broadcast-templates/<int:pk>/',
         BroadcastTemplateDetail.as_view(),
         name='show-broadcast-template'),
    path('broadcast-templates/<int:pk>/edit',
         BroadcastTemplateUpdate.as_view(),
         name='update-broadcast-template'),
    path('broadcast-templates/<int:pk>/delete/',
         BroadcastTemplateDelete.as_view(),
         name='delete-broadcast-template'),

    path('broadcast-templates/<int:bctmpl_pk>/add-regular-item',
         BroadcastTemplateRegularItemCreate.as_view(),
         name='add-regular-item-to-broadcast-template'),

    path('broadcast-template-regular-item/<int:pk>/edit',
         BroadcastTemplateRegularItemUpdate.as_view(),
         name='edit-regular-item-broadcast-template'),

    path('broadcast-template-regular-item/<int:pk>/delete',
         BroadcastTemplateRegularItemDelete.as_view(),
         name='delete-regular-item-from-broadcast-template'),
    path('broadcast-template-regular-item/<int:pk>/<action>',
         move_regular_item_of_broadcast_template, name='move-broadcast-template-regular-item'),


    path('broadcast-templates/<int:bctmpl_pk>/add-repeat',
         BroadcastRepeatTemplateCreate.as_view(),
         name='add-repeat-to-broadcast-template'),
    path('broadcast-repeat-template/<int:pk>/edit',
         BroadcastRepeatTemplateUpdate.as_view(),
         name='edit-repeat-from-broadcast-template'),
    path('broadcast-repeat-template/<int:pk>/delete',
         BroadcastRepeatTemplateDelete.as_view(),
         name='delete-repeat-from-broadcast-template'),

    path('broadcasts/create-from-templates',
         createBroadcastsFromTemplate,
         name='create-broadcasts-from-templates'),

    path('radioprograms/',
         RadioProgramList.as_view(),
         name='list-radio-programs'),
    path('radioprograms/<int:pk>/',
         RadioProgramDetail.as_view(),
         name='show-radio-program'),
]
