import os
from setuptools import setup, find_packages


# parse dependencies from requirements.txt
def get_requirements():
    with open('requirements.txt') as f:
        return [
            line.split('#')[0].strip() for line in f.read().splitlines()
            if not line.strip().startswith('#')
        ]


def get_readme():
    with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as f:
        return f.read()


def get_version():
    with open(os.path.join(os.path.dirname(__file__), 'VERSION')) as f:
        return f.read().strip()


def include_recursive(directory):
    result = {}
    for dirpath, dirnames, filenames in os.walk(directory):
        if filenames:
            result[dirpath] = [os.path.join(dirpath, fname) for fname in filenames]
    return tuple(result.items())


setup(
    name='lohrothek-backend',
    version=get_version(),
    description='TODO',
    long_description=get_readme(),
    url='https://stadtgestalten.org/lohrothek/',
    author='lohrothek Maintainers',
    author_email='TODO',
    packages=find_packages(exclude=['build.*', 'debian.*']),
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: End Users/Desktop',
        'Development Status :: 5 - Production/Stable',
        'Natural Language :: German',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Programming Language :: Python :: 3',
        'Environment :: Web Environment',
        'Topic :: Internet :: WWW/HTTP',
    ],
    install_requires=get_requirements(),
    data_files=(
        ('.', (
            'manage.py', 'README.md', 'LICENSE', 'requirements.txt',
        )),
    ),
)
